@@ -1,22 +1,12 @@
-// Copyright 2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 // These functions are used by macro expansion for bug! and span_bug!
 
-use ty::tls;
+use crate::ty::tls;
 use std::fmt;
 use syntax_pos::{Span, MultiSpan};
 
 #[cold]
 #[inline(never)]
-pub fn bug_fmt(file: &'static str, line: u32, args: fmt::Arguments) -> ! {
+pub fn bug_fmt(file: &'static str, line: u32, args: fmt::Arguments<'_>) -> ! {
     // this wrapper mostly exists so I don't have to write a fully
     // qualified path of None::<Span> inside the bug!() macro definition
     opt_span_bug_fmt(file, line, None::<Span>, args);
@@ -28,7 +18,7 @@ pub fn span_bug_fmt<S: Into<MultiSpan>>(
     file: &'static str,
     line: u32,
     span: S,
-    args: fmt::Arguments,
+    args: fmt::Arguments<'_>,
 ) -> ! {
     opt_span_bug_fmt(file, line, Some(span), args);
 }
@@ -37,7 +27,7 @@ fn opt_span_bug_fmt<S: Into<MultiSpan>>(
     file: &'static str,
     line: u32,
     span: Option<S>,
-    args: fmt::Arguments,
+    args: fmt::Arguments<'_>,
 ) -> ! {
     tls::with_opt(move |tcx| {
         let msg = format!("{}:{}: {}", file, line, args);
