@@ -1,24 +1,15 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
-use crate::rustc::hir;
-use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
+use crate::utils::span_lint_and_sugg;
 use if_chain::if_chain;
-use crate::rustc::ty::TyKind;
+use rustc::hir;
+use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use rustc::ty;
+use rustc::{declare_tool_lint, lint_array};
+use rustc_errors::Applicability;
 use std::f32;
 use std::f64;
 use std::fmt;
-use crate::syntax::ast::*;
-use crate::syntax_pos::symbol::Symbol;
-use crate::utils::span_lint_and_sugg;
+use syntax::ast::*;
+use syntax_pos::symbol::Symbol;
 
 /// **What it does:** Checks for float literals with a precision greater
 /// than that supported by the underlying type
@@ -31,12 +22,12 @@ use crate::utils::span_lint_and_sugg;
 ///
 /// ```rust
 /// // Bad
-///    let v: f32 = 0.123_456_789_9;
-///    println!("{}", v); //  0.123_456_789
+/// let v: f32 = 0.123_456_789_9;
+/// println!("{}", v); //  0.123_456_789
 ///
 /// // Good
-///    let v: f64 = 0.123_456_789_9;
-///    println!("{}", v); //  0.123_456_789_9
+/// let v: f64 = 0.123_456_789_9;
+/// println!("{}", v); //  0.123_456_789_9
 /// ```
 declare_clippy_lint! {
     pub EXCESSIVE_PRECISION,
@@ -50,13 +41,17 @@ impl LintPass for ExcessivePrecision {
     fn get_lints(&self) -> LintArray {
         lint_array!(EXCESSIVE_PRECISION)
     }
+
+    fn name(&self) -> &'static str {
+        "ExcessivePrecision"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for ExcessivePrecision {
     fn check_expr(&mut self, cx: &LateContext<'a, 'tcx>, expr: &'tcx hir::Expr) {
         if_chain! {
             let ty = cx.tables.expr_ty(expr);
-            if let TyKind::Float(fty) = ty.sty;
+            if let ty::Float(fty) = ty.sty;
             if let hir::ExprKind::Lit(ref lit) = expr.node;
             if let LitKind::Float(sym, _) | LitKind::FloatUnsuffixed(sym) = lit.node;
             if let Some(sugg) = self.check(sym, fty);
@@ -68,6 +63,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for ExcessivePrecision {
                     "float has excessive precision",
                     "consider changing the type or truncating it to",
                     sugg,
+                    Applicability::MachineApplicable,
                 );
             }
         }
@@ -80,7 +76,7 @@ impl ExcessivePrecision {
         let max = max_digits(fty);
         let sym_str = sym.as_str();
         if dot_zero_exclusion(&sym_str) {
-            return None
+            return None;
         }
         // Try to bail out if the float is for sure fine.
         // If its within the 2 decimal digits of being out of precision we
@@ -108,15 +104,12 @@ impl ExcessivePrecision {
     }
 }
 
-#[allow(clippy::doc_markdown)]
 /// Should we exclude the float because it has a `.0` or `.` suffix
-/// Ex 1_000_000_000.0
-/// Ex 1_000_000_000.
+/// Ex `1_000_000_000.0`
+/// Ex `1_000_000_000.`
 fn dot_zero_exclusion(s: &str) -> bool {
     if let Some(after_dec) = s.split('.').nth(1) {
-        let mut decpart = after_dec
-            .chars()
-            .take_while(|c| *c != 'e' || *c != 'E');
+        let mut decpart = after_dec.chars().take_while(|c| *c != 'e' || *c != 'E');
 
         match decpart.next() {
             Some('0') => decpart.count() == 0,
@@ -167,7 +160,9 @@ impl FloatFormat {
             .unwrap_or(FloatFormat::Normal)
     }
     fn format<T>(&self, f: T) -> String
-    where T: fmt::UpperExp + fmt::LowerExp + fmt::Display {
+    where
+        T: fmt::UpperExp + fmt::LowerExp + fmt::Display,
+    {
         match self {
             FloatFormat::LowerExp => format!("{:e}", f),
             FloatFormat::UpperExp => format!("{:E}", f),
