@@ -1,22 +1,12 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
 //! lint on inherent implementations
 
-use crate::rustc::hir::*;
-use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
-use crate::rustc_data_structures::fx::FxHashMap;
-use std::default::Default;
-use crate::syntax_pos::Span;
 use crate::utils::span_lint_and_then;
+use rustc::hir::*;
+use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use rustc::{declare_tool_lint, lint_array};
+use rustc_data_structures::fx::FxHashMap;
+use std::default::Default;
+use syntax_pos::Span;
 
 /// **What it does:** Checks for multiple inherent implementations of a struct
 ///
@@ -56,7 +46,9 @@ pub struct Pass {
 
 impl Default for Pass {
     fn default() -> Self {
-        Self { impls: FxHashMap::default() }
+        Self {
+            impls: FxHashMap::default(),
+        }
     }
 }
 
@@ -64,6 +56,10 @@ impl LintPass for Pass {
     fn get_lints(&self) -> LintArray {
         lint_array!(MULTIPLE_INHERENT_IMPL)
     }
+
+    fn name(&self) -> &'static str {
+        "MultipleInherientImpl"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Pass {
@@ -88,11 +84,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Pass {
                 let mut impl_spans = impls
                     .iter()
                     .filter_map(|impl_def| self.impls.get(impl_def))
-                    .filter_map(|(span, generics)| if generics.params.len() == 0 {
-                        Some(span)
-                    } else {
-                        None
-                    });
+                    .filter_map(|(span, generics)| if generics.params.len() == 0 { Some(span) } else { None });
                 if let Some(initial_span) = impl_spans.nth(0) {
                     impl_spans.for_each(|additional_span| {
                         span_lint_and_then(
@@ -101,10 +93,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Pass {
                             *additional_span,
                             "Multiple implementations of this structure",
                             |db| {
-                                db.span_note(
-                                    *initial_span,
-                                    "First implementation here",
-                                );
+                                db.span_note(*initial_span, "First implementation here");
                             },
                         )
                     })
