@@ -1,12 +1,13 @@
-use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
-use if_chain::if_chain;
-use crate::rustc::ty::{self, Ty};
-use crate::rustc::hir::*;
-use std::borrow::Cow;
-use crate::syntax::ast;
 use crate::utils::{last_path_segment, match_def_path, paths, snippet, span_lint, span_lint_and_then};
 use crate::utils::{opt_def_id, sugg};
+use if_chain::if_chain;
+use rustc::hir::*;
+use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use rustc::ty::{self, Ty};
+use rustc::{declare_tool_lint, lint_array};
+use rustc_errors::Applicability;
+use std::borrow::Cow;
+use syntax::ast;
 
 /// **What it does:** Checks for transmutes that can't ever be correct on any
 /// architecture.
@@ -54,7 +55,7 @@ declare_clippy_lint! {
 /// **Example:**
 /// ```rust
 /// core::intrinsics::transmute(t) // where the result type is the same as
-/// // `*t` or `&t`'s
+///                                // `*t` or `&t`'s
 /// ```
 declare_clippy_lint! {
     pub CROSSPOINTER_TRANSMUTE,
@@ -71,6 +72,7 @@ declare_clippy_lint! {
 /// **Example:**
 /// ```rust
 /// let _: &T = std::mem::transmute(p); // where p: *const T
+///
 /// // can be written:
 /// let _: &T = &*p;
 /// ```
@@ -98,6 +100,7 @@ declare_clippy_lint! {
 /// **Example:**
 /// ```rust
 /// let _: char = std::mem::transmute(x); // where x: u32
+///
 /// // should be:
 /// let _ = std::char::from_u32(x).unwrap();
 /// ```
@@ -125,6 +128,7 @@ declare_clippy_lint! {
 /// **Example:**
 /// ```rust
 /// let _: &str = std::mem::transmute(b); // where b: &[u8]
+///
 /// // should be:
 /// let _ = std::str::from_utf8(b).unwrap();
 /// ```
@@ -143,6 +147,7 @@ declare_clippy_lint! {
 /// **Example:**
 /// ```rust
 /// let _: bool = std::mem::transmute(x); // where x: u8
+///
 /// // should be:
 /// let _: bool = x != 0;
 /// ```
@@ -154,13 +159,15 @@ declare_clippy_lint! {
 
 /// **What it does:** Checks for transmutes from an integer to a float.
 ///
-/// **Why is this bad?** This might result in an invalid in-memory representation of a float.
+/// **Why is this bad?** Transmutes are dangerous and error-prone, whereas `from_bits` is intuitive
+/// and safe.
 ///
 /// **Known problems:** None.
 ///
 /// **Example:**
 /// ```rust
 /// let _: f32 = std::mem::transmute(x); // where x: u32
+///
 /// // should be:
 /// let _: f32 = f32::from_bits(x);
 /// ```
@@ -213,9 +220,14 @@ impl LintPass for Transmute {
             TRANSMUTE_INT_TO_FLOAT,
         )
     }
+
+    fn name(&self) -> &'static str {
+        "Transmute"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
+    #[allow(clippy::similar_names, clippy::too_many_lines)]
     fn check_expr(&mut self, cx: &LateContext<'a, 'tcx>, e: &'tcx Expr) {
         if let ExprKind::Call(ref path_expr, ref args) = e.node {
             if let ExprKind::Path(ref qpath) = path_expr.node {
@@ -236,33 +248,43 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                 USELESS_TRANSMUTE,
                                 e.span,
                                 "transmute from a reference to a pointer",
-                                |db| if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
-                                    let rty_and_mut = ty::TypeAndMut { ty: rty, mutbl: rty_mutbl };
+                                |db| {
+                                    if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
+                                        let rty_and_mut = ty::TypeAndMut {
+                                            ty: rty,
+                                            mutbl: rty_mutbl,
+                                        };
 
-                                    let sugg = if ptr_ty == rty_and_mut {
-                                        arg.as_ty(to_ty)
-                                    } else {
-                                        arg.as_ty(cx.tcx.mk_ptr(rty_and_mut)).as_ty(to_ty)
-                                    };
+                                        let sugg = if ptr_ty == rty_and_mut {
+                                            arg.as_ty(to_ty)
+                                        } else {
+                                            arg.as_ty(cx.tcx.mk_ptr(rty_and_mut)).as_ty(to_ty)
+                                        };
 
-                                    db.span_suggestion(e.span, "try", sugg.to_string());
+                                        db.span_suggestion(e.span, "try", sugg.to_string(), Applicability::Unspecified);
+                                    }
                                 },
                             ),
-                            (&ty::Int(_), &ty::RawPtr(_)) | (&ty::Uint(_), &ty::RawPtr(_)) => {
-                                span_lint_and_then(
-                                    cx,
-                                    USELESS_TRANSMUTE,
-                                    e.span,
-                                    "transmute from an integer to a pointer",
-                                    |db| if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
-                                        db.span_suggestion(e.span, "try", arg.as_ty(&to_ty.to_string()).to_string());
-                                    },
-                                )
-                            },
-                            (&ty::Float(_), &ty::Ref(..)) |
-                            (&ty::Float(_), &ty::RawPtr(_)) |
-                            (&ty::Char, &ty::Ref(..)) |
-                            (&ty::Char, &ty::RawPtr(_)) => span_lint(
+                            (&ty::Int(_), &ty::RawPtr(_)) | (&ty::Uint(_), &ty::RawPtr(_)) => span_lint_and_then(
+                                cx,
+                                USELESS_TRANSMUTE,
+                                e.span,
+                                "transmute from an integer to a pointer",
+                                |db| {
+                                    if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
+                                        db.span_suggestion(
+                                            e.span,
+                                            "try",
+                                            arg.as_ty(&to_ty.to_string()).to_string(),
+                                            Applicability::Unspecified,
+                                        );
+                                    }
+                                },
+                            ),
+                            (&ty::Float(_), &ty::Ref(..))
+                            | (&ty::Float(_), &ty::RawPtr(_))
+                            | (&ty::Char, &ty::Ref(..))
+                            | (&ty::Char, &ty::RawPtr(_)) => span_lint(
                                 cx,
                                 WRONG_TRANSMUTE,
                                 e.span,
@@ -274,8 +296,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                 e.span,
                                 &format!(
                                     "transmute from a type (`{}`) to the type that it points to (`{}`)",
-                                    from_ty,
-                                    to_ty
+                                    from_ty, to_ty
                                 ),
                             ),
                             (_, &ty::RawPtr(to_ptr)) if to_ptr.ty == from_ty => span_lint(
@@ -284,8 +305,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                 e.span,
                                 &format!(
                                     "transmute from a type (`{}`) to a pointer to that type (`{}`)",
-                                    from_ty,
-                                    to_ty
+                                    from_ty, to_ty
                                 ),
                             ),
                             (&ty::RawPtr(from_pty), &ty::Ref(_, to_ref_ty, mutbl)) => span_lint_and_then(
@@ -295,8 +315,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                 &format!(
                                     "transmute from a pointer type (`{}`) to a reference type \
                                      (`{}`)",
-                                    from_ty,
-                                    to_ty
+                                    from_ty, to_ty
                                 ),
                                 |db| {
                                     let arg = sugg::Sugg::hir(cx, &args[0], "..");
@@ -312,29 +331,36 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                         arg.as_ty(&format!("{} {}", cast, get_type_snippet(cx, qpath, to_ref_ty)))
                                     };
 
-                                    db.span_suggestion(e.span, "try", sugg::make_unop(deref, arg).to_string());
-                                },
-                            ),
-                            (&ty::Int(ast::IntTy::I32), &ty::Char) |
-                            (&ty::Uint(ast::UintTy::U32), &ty::Char) => span_lint_and_then(
-                                cx,
-                                TRANSMUTE_INT_TO_CHAR,
-                                e.span,
-                                &format!("transmute from a `{}` to a `char`", from_ty),
-                                |db| {
-                                    let arg = sugg::Sugg::hir(cx, &args[0], "..");
-                                    let arg = if let ty::Int(_) = from_ty.sty {
-                                        arg.as_ty(ty::Uint(ast::UintTy::U32))
-                                    } else {
-                                        arg
-                                    };
                                     db.span_suggestion(
                                         e.span,
-                                        "consider using",
-                                        format!("std::char::from_u32({}).unwrap()", arg.to_string()),
+                                        "try",
+                                        sugg::make_unop(deref, arg).to_string(),
+                                        Applicability::Unspecified,
                                     );
                                 },
                             ),
+                            (&ty::Int(ast::IntTy::I32), &ty::Char) | (&ty::Uint(ast::UintTy::U32), &ty::Char) => {
+                                span_lint_and_then(
+                                    cx,
+                                    TRANSMUTE_INT_TO_CHAR,
+                                    e.span,
+                                    &format!("transmute from a `{}` to a `char`", from_ty),
+                                    |db| {
+                                        let arg = sugg::Sugg::hir(cx, &args[0], "..");
+                                        let arg = if let ty::Int(_) = from_ty.sty {
+                                            arg.as_ty(ty::Uint(ast::UintTy::U32))
+                                        } else {
+                                            arg
+                                        };
+                                        db.span_suggestion(
+                                            e.span,
+                                            "consider using",
+                                            format!("std::char::from_u32({}).unwrap()", arg.to_string()),
+                                            Applicability::Unspecified,
+                                        );
+                                    },
+                                )
+                            },
                             (&ty::Ref(_, ty_from, from_mutbl), &ty::Ref(_, ty_to, to_mutbl)) => {
                                 if_chain! {
                                     if let (&ty::Slice(slice_ty), &ty::Str) = (&ty_from.sty, &ty_to.sty);
@@ -361,6 +387,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                                         postfix,
                                                         snippet(cx, args[0].span, ".."),
                                                     ),
+                                                    Applicability::Unspecified,
                                                 );
                                             }
                                         )
@@ -372,15 +399,25 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                                 e.span,
                                                 "transmute from a reference to a reference",
                                                 |db| if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
-                                                    let ty_from_and_mut = ty::TypeAndMut { ty: ty_from, mutbl: from_mutbl };
+                                                    let ty_from_and_mut = ty::TypeAndMut {
+                                                        ty: ty_from,
+                                                        mutbl: from_mutbl
+                                                    };
                                                     let ty_to_and_mut = ty::TypeAndMut { ty: ty_to, mutbl: to_mutbl };
-                                                    let sugg_paren = arg.as_ty(cx.tcx.mk_ptr(ty_from_and_mut)).as_ty(cx.tcx.mk_ptr(ty_to_and_mut));
+                                                    let sugg_paren = arg
+                                                        .as_ty(cx.tcx.mk_ptr(ty_from_and_mut))
+                                                        .as_ty(cx.tcx.mk_ptr(ty_to_and_mut));
                                                     let sugg = if to_mutbl == Mutability::MutMutable {
                                                         sugg_paren.mut_addr_deref()
                                                     } else {
                                                         sugg_paren.addr_deref()
                                                     };
-                                                    db.span_suggestion(e.span, "try", sugg.to_string());
+                                                    db.span_suggestion(
+                                                        e.span,
+                                                        "try",
+                                                        sugg.to_string(),
+                                                        Applicability::Unspecified,
+                                                    );
                                                 },
                                             )
                                         }
@@ -392,9 +429,11 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                 TRANSMUTE_PTR_TO_PTR,
                                 e.span,
                                 "transmute from a pointer to a pointer",
-                                |db| if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
-                                    let sugg = arg.as_ty(cx.tcx.mk_ptr(to_ty));
-                                    db.span_suggestion(e.span, "try", sugg.to_string());
+                                |db| {
+                                    if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
+                                        let sugg = arg.as_ty(cx.tcx.mk_ptr(to_ty));
+                                        db.span_suggestion(e.span, "try", sugg.to_string(), Applicability::Unspecified);
+                                    }
                                 },
                             ),
                             (&ty::Int(ast::IntTy::I8), &ty::Bool) | (&ty::Uint(ast::UintTy::U8), &ty::Bool) => {
@@ -410,36 +449,34 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                             e.span,
                                             "consider using",
                                             sugg::make_binop(ast::BinOpKind::Ne, &arg, &zero).to_string(),
+                                            Applicability::Unspecified,
                                         );
                                     },
                                 )
                             },
-                            (&ty::Int(_), &ty::Float(_)) | (&ty::Uint(_), &ty::Float(_)) => {
-                                span_lint_and_then(
-                                    cx,
-                                    TRANSMUTE_INT_TO_FLOAT,
-                                    e.span,
-                                    &format!("transmute from a `{}` to a `{}`", from_ty, to_ty),
-                                    |db| {
-                                        let arg = sugg::Sugg::hir(cx, &args[0], "..");
-                                        let arg = if let ty::Int(int_ty) = from_ty.sty {
-                                            arg.as_ty(format!(
-                                                "u{}",
-                                                int_ty
-                                                    .bit_width()
-                                                    .map_or_else(|| "size".to_string(), |v| v.to_string())
-                                            ))
-                                        } else {
-                                            arg
-                                        };
-                                        db.span_suggestion(
-                                            e.span,
-                                            "consider using",
-                                            format!("{}::from_bits({})", to_ty, arg.to_string()),
-                                        );
-                                    },
-                                )
-                            },
+                            (&ty::Int(_), &ty::Float(_)) | (&ty::Uint(_), &ty::Float(_)) => span_lint_and_then(
+                                cx,
+                                TRANSMUTE_INT_TO_FLOAT,
+                                e.span,
+                                &format!("transmute from a `{}` to a `{}`", from_ty, to_ty),
+                                |db| {
+                                    let arg = sugg::Sugg::hir(cx, &args[0], "..");
+                                    let arg = if let ty::Int(int_ty) = from_ty.sty {
+                                        arg.as_ty(format!(
+                                            "u{}",
+                                            int_ty.bit_width().map_or_else(|| "size".to_string(), |v| v.to_string())
+                                        ))
+                                    } else {
+                                        arg
+                                    };
+                                    db.span_suggestion(
+                                        e.span,
+                                        "consider using",
+                                        format!("{}::from_bits({})", to_ty, arg.to_string()),
+                                        Applicability::Unspecified,
+                                    );
+                                },
+                            ),
                             _ => return,
                         };
                     }
@@ -461,7 +498,7 @@ fn get_type_snippet(cx: &LateContext<'_, '_>, path: &QPath, to_ref_ty: Ty<'_>) -
         if !params.parenthesized;
         if let Some(to_ty) = params.args.iter().filter_map(|arg| match arg {
             GenericArg::Type(ty) => Some(ty),
-            GenericArg::Lifetime(_) => None,
+            _ => None,
         }).nth(1);
         if let TyKind::Rptr(_, ref to_ty) = to_ty.node;
         then {
