@@ -1,6 +1,8 @@
-#![feature(tool_lints)]
-#[warn(clippy::cast_lossless)]
-#[allow(clippy::no_effect, clippy::unnecessary_operation)]
+// run-rustfix
+
+#![allow(clippy::no_effect, clippy::unnecessary_operation, dead_code)]
+#![warn(clippy::cast_lossless)]
+
 fn main() {
     // Test clippy::cast_lossless with casts to integer types
     1i8 as i16;
@@ -22,3 +24,10 @@ fn main() {
     1u32 as i64;
     1u32 as u64;
 }
+
+// The lint would suggest using `f64::from(input)` here but the `XX::from` function is not const,
+// so we skip the lint if the expression is in a const fn.
+// See #3656
+const fn abc(input: u16) -> u32 {
+    input as u32
+}
