@@ -1,6 +1,10 @@
-#![feature(tool_lints)]
-
-#[warn(clippy::cast_precision_loss, clippy::cast_possible_truncation, clippy::cast_sign_loss, clippy::cast_possible_wrap, clippy::cast_lossless)]
+#[warn(
+    clippy::cast_precision_loss,
+    clippy::cast_possible_truncation,
+    clippy::cast_sign_loss,
+    clippy::cast_possible_wrap,
+    clippy::cast_lossless
+)]
 #[allow(clippy::no_effect, clippy::unnecessary_operation)]
 fn main() {
     // Casting from *size
