@@ -1,6 +1,3 @@
-#![feature(tool_lints)]
-
-
 #![warn(clippy::all)]
 #![allow(unused_variables)]
 #![allow(unused_assignments)]
@@ -10,7 +7,11 @@
 fn foo() -> bool { true }
 
 fn main() {
-    // weird `else if` formatting:
+    // weird `else` formatting:
+    if foo() {
+    } {
+    }
+
     if foo() {
     } if foo() {
     }
@@ -35,6 +36,17 @@ fn main() {
         let _ = 0;
     };
 
+    if foo() {
+    } else
+    {
+    }
+
+    if foo() {
+    }
+    else
+    {
+    }
+
     if foo() {
     } else
     if foo() { // the span of the above error should continue here
@@ -47,6 +59,20 @@ fn main() {
     }
 
     // those are ok:
+    if foo() {
+    }
+    {
+    }
+
+    if foo() {
+    } else {
+    }
+
+    if foo() {
+    }
+    else {
+    }
+
     if foo() {
     }
     if foo() {
@@ -102,4 +128,16 @@ fn main() {
         1 + 2, 3 +
         4, 5 + 6,
     ];
+
+    // don't lint for bin op without unary equiv
+    // issue 3244
+    vec![
+        1
+        / 2,
+    ];
+    // issue 3396
+    vec![
+        true
+        | false,
+    ];
 }
