@@ -1,5 +1,3 @@
-#![feature(tool_lints)]
-
 #![deny(clippy::useless_asref)]
 #![allow(clippy::trivially_copy_pass_by_ref)]
 use std::fmt::Debug;
@@ -8,7 +6,9 @@ struct FakeAsRef;
 
 #[allow(clippy::should_implement_trait)]
 impl FakeAsRef {
-    fn as_ref(&self) -> &Self { self }
+    fn as_ref(&self) -> &Self {
+        self
+    }
 }
 
 struct MoreRef;
@@ -19,14 +19,22 @@ impl<'a, 'b, 'c> AsRef<&'a &'b &'c MoreRef> for MoreRef {
     }
 }
 
-fn foo_rstr(x: &str) { println!("{:?}", x); }
-fn foo_rslice(x: &[i32]) { println!("{:?}", x); }
-fn foo_mrslice(x: &mut [i32]) { println!("{:?}", x); }
-fn foo_rrrrmr(_: &&&&MoreRef) { println!("so many refs"); }
+fn foo_rstr(x: &str) {
+    println!("{:?}", x);
+}
+fn foo_rslice(x: &[i32]) {
+    println!("{:?}", x);
+}
+fn foo_mrslice(x: &mut [i32]) {
+    println!("{:?}", x);
+}
+fn foo_rrrrmr(_: &&&&MoreRef) {
+    println!("so many refs");
+}
 
 fn not_ok() {
     let rstr: &str = "hello";
-    let mut mrslice: &mut [i32] = &mut [1,2,3];
+    let mut mrslice: &mut [i32] = &mut [1, 2, 3];
 
     {
         let rslice: &[i32] = &*mrslice;
@@ -65,8 +73,8 @@ fn not_ok() {
 
 fn ok() {
     let string = "hello".to_owned();
-    let mut arr = [1,2,3];
-    let mut vec = vec![1,2,3];
+    let mut arr = [1, 2, 3];
+    let mut vec = vec![1, 2, 3];
 
     {
         foo_rstr(string.as_ref());
@@ -99,8 +107,12 @@ fn ok() {
     generic_ok(&mut arr);
 }
 
-fn foo_mrt<T: Debug + ?Sized>(t: &mut T) { println!("{:?}", t); }
-fn foo_rt<T: Debug + ?Sized>(t: &T) { println!("{:?}", t); }
+fn foo_mrt<T: Debug + ?Sized>(t: &mut T) {
+    println!("{:?}", t);
+}
+fn foo_rt<T: Debug + ?Sized>(t: &T) {
+    println!("{:?}", t);
+}
 
 fn generic_not_ok<T: AsMut<T> + AsRef<T> + Debug + ?Sized>(mrt: &mut T) {
     foo_mrt(mrt.as_mut());
