@@ -1,16 +1,3 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
-
-
-
 #![warn(clippy::while_let_loop, clippy::empty_loop, clippy::while_let_on_iterator)]
 #![allow(dead_code, clippy::never_loop, unused, clippy::cyclomatic_complexity)]
 
@@ -20,10 +7,11 @@ fn main() {
         if let Some(_x) = y {
             let _v = 1;
         } else {
-            break
+            break;
         }
     }
-    loop { // no error, break is not in else clause
+    loop {
+        // no error, break is not in else clause
         if let Some(_x) = y {
             let _v = 1;
         }
@@ -32,13 +20,13 @@ fn main() {
     loop {
         match y {
             Some(_x) => true,
-            None => break
+            None => break,
         };
     }
     loop {
         let x = match y {
             Some(x) => x,
-            None => break
+            None => break,
         };
         let _x = x;
         let _str = "foo";
@@ -48,19 +36,25 @@ fn main() {
             Some(x) => x,
             None => break,
         };
-        { let _a = "bar"; };
-        { let _b = "foobar"; }
+        {
+            let _a = "bar";
+        };
+        {
+            let _b = "foobar";
+        }
     }
-    loop { // no error, else branch does something other than break
+    loop {
+        // no error, else branch does something other than break
         match y {
             Some(_x) => true,
             _ => {
                 let _z = 1;
                 break;
-            }
+            },
         };
     }
-    while let Some(x) = y { // no error, obviously
+    while let Some(x) = y {
+        // no error, obviously
         println!("{}", x);
     }
 
@@ -68,7 +62,7 @@ fn main() {
     loop {
         let (e, l) = match "".split_whitespace().next() {
             Some(word) => (word.is_empty(), word.len()),
-            None => break
+            None => break,
         };
 
         let _ = (e, l);
@@ -91,7 +85,8 @@ fn main() {
     while let None = iter.next() {} // this is fine (if nonsensical)
 
     let mut iter = 1..20;
-    if let Some(x) = iter.next() { // also fine
+    if let Some(x) = iter.next() {
+        // also fine
         println!("{}", x)
     }
 
@@ -109,7 +104,9 @@ fn main() {
 
     // or this
     let mut iter = 1u32..20;
-    while let Some(x) = iter.next() {break;}
+    while let Some(x) = iter.next() {
+        break;
+    }
     println!("Remaining iter {:?}", iter);
 
     // or this
@@ -128,7 +125,7 @@ fn no_panic<T>(slice: &[T]) {
     loop {
         let _ = match iter.next() {
             Some(ele) => ele,
-            None => break
+            None => break,
         };
         loop {}
     }
@@ -143,8 +140,8 @@ fn issue1017() {
             Err(_) => len = 0,
             Ok(length) => {
                 len = length;
-                break
-            }
+                break;
+            },
         }
     }
 }
@@ -155,20 +152,17 @@ fn refutable() {
     let mut b = a.iter();
 
     // consume all the 42s
-    while let Some(&42) = b.next() {
-    }
+    while let Some(&42) = b.next() {}
 
     let a = [(1, 2, 3)];
     let mut b = a.iter();
 
-    while let Some(&(1, 2, 3)) = b.next() {
-    }
+    while let Some(&(1, 2, 3)) = b.next() {}
 
     let a = [Some(42)];
     let mut b = a.iter();
 
-    while let Some(&None) = b.next() {
-    }
+    while let Some(&None) = b.next() {}
 
     /* This gives “refutable pattern in `for` loop binding: `&_` not covered”
     for &42 in b {}
@@ -177,20 +171,22 @@ fn refutable() {
     // */
 
     let mut y = a.iter();
-    loop { // x is reused, so don't lint here
-        while let Some(v) = y.next() {
-        }
+    loop {
+        // x is reused, so don't lint here
+        while let Some(v) = y.next() {}
     }
 
     let mut y = a.iter();
     for _ in 0..2 {
-        while let Some(v) = y.next() { // y is reused, don't lint
+        while let Some(v) = y.next() {
+            // y is reused, don't lint
         }
     }
 
     loop {
         let mut y = a.iter();
-        while let Some(v) = y.next() { // use a for loop here
+        while let Some(v) = y.next() {
+            // use a for loop here
         }
     }
 
@@ -220,4 +216,14 @@ fn refutable() {
     while let Some(..) = values.iter().next() {
         values.remove(&1);
     }
+
+    // Issue 3670
+    {
+        let array = [Some(0), None, Some(1)];
+        let mut iter = array.iter();
+
+        while let Some(elem) = iter.next() {
+            let _ = elem.or_else(|| *iter.next()?);
+        }
+    }
 }
