@@ -8,11 +8,11 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-extern crate env_logger;
-extern crate getopts;
+use env_logger;
+
 #[macro_use]
 extern crate log;
-extern crate rustfmt_nightly as rustfmt;
+use rustfmt_nightly as rustfmt;
 
 use std::env;
 use std::io::stdout;
@@ -22,7 +22,7 @@ use std::str::FromStr;
 
 use getopts::{Matches, Options};
 
-use rustfmt::{load_config, CliOptions, Input, Session};
+use crate::rustfmt::{load_config, CliOptions, Input, Session};
 
 fn prune_files(files: Vec<&str>) -> Vec<&str> {
     let prefixes: Vec<_> = files
