@@ -8,28 +8,17 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
+#![deny(rust_2018_idioms)]
+
 #[macro_use]
 extern crate derive_new;
-extern crate atty;
-extern crate bytecount;
-extern crate diff;
-extern crate failure;
-extern crate itertools;
 #[cfg(test)]
 #[macro_use]
 extern crate lazy_static;
 #[macro_use]
 extern crate log;
-extern crate regex;
-extern crate rustc_target;
-extern crate serde;
 #[macro_use]
 extern crate serde_derive;
-extern crate serde_json;
-extern crate syntax;
-extern crate syntax_pos;
-extern crate toml;
-extern crate unicode_segmentation;
 
 use std::cell::RefCell;
 use std::collections::HashMap;
@@ -39,15 +28,17 @@ use std::mem;
 use std::panic;
 use std::path::PathBuf;
 use std::rc::Rc;
+
+use failure::Fail;
 use syntax::ast;
 
-use comment::LineClasses;
-use failure::Fail;
-use formatting::{FormatErrorMap, FormattingError, ReportedErrors, SourceFile};
-use issues::Issue;
-use shape::Indent;
+use crate::comment::LineClasses;
+use crate::formatting::{FormatErrorMap, FormattingError, ReportedErrors, SourceFile};
+use crate::issues::Issue;
+use crate::shape::Indent;
+use crate::utils::indent_next_line;
 
-pub use config::{
+pub use crate::config::{
     load_config, CliOptions, Color, Config, Edition, EmitMode, FileLines, FileName, NewlineStyle,
     Range, Verbosity,
 };
@@ -259,7 +250,7 @@ impl FormatReport {
     /// fancy output.
     pub fn fancy_print(
         &self,
-        mut t: Box<term::Terminal<Output = io::Stderr>>,
+        mut t: Box<dyn term::Terminal<Output = io::Stderr>>,
     ) -> Result<(), term::Error> {
         for (file, errors) in &self.internal.borrow().0 {
             for error in errors {
@@ -330,7 +321,7 @@ impl FormatReport {
 
 impl fmt::Display for FormatReport {
     // Prints all the formatting errors.
-    fn fmt(&self, fmt: &mut fmt::Formatter) -> Result<(), fmt::Error> {
+    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> Result<(), fmt::Error> {
         for (file, errors) in &self.internal.borrow().0 {
             for error in errors {
                 let prefix_space_len = error.line.to_string().len();
@@ -434,7 +425,7 @@ fn format_code_block(code_snippet: &str, config: &Config) -> Option<FormattedSni
             }
             result.push_str(&line);
             result.push('\n');
-            need_indent = !kind.is_string() || line.ends_with('\\');
+            need_indent = indent_next_line(kind, &line, config);
         }
         result.push('}');
         result
@@ -495,7 +486,7 @@ fn format_code_block(code_snippet: &str, config: &Config) -> Option<FormattedSni
             line
         };
         result.push_str(trimmed_line);
-        is_indented = !kind.is_string() || line.ends_with('\\');
+        is_indented = indent_next_line(kind, line, config);
     }
     Some(FormattedSnippet {
         snippet: result,
@@ -504,7 +495,7 @@ fn format_code_block(code_snippet: &str, config: &Config) -> Option<FormattedSni
 }
 
 /// A session is a run of rustfmt across a single or multiple inputs.
-pub struct Session<'b, T: Write + 'b> {
+pub struct Session<'b, T: Write> {
     pub config: Config,
     pub out: Option<&'b mut T>,
     pub(crate) errors: ReportedErrors,
