@@ -8,8 +8,6 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-extern crate assert_cli;
-
 use std::collections::{HashMap, HashSet};
 use std::env;
 use std::fs;
@@ -17,13 +15,14 @@ use std::io::{self, BufRead, BufReader, Read, Write};
 use std::iter::{Enumerate, Peekable};
 use std::mem;
 use std::path::{Path, PathBuf};
+use std::process::{Command, Stdio};
 use std::str::Chars;
 
-use config::{Color, Config, EmitMode, FileName, ReportTactic};
-use formatting::{ModifiedChunk, SourceFile};
-use rustfmt_diff::{make_diff, print_diff, DiffLine, Mismatch, OutputWriter};
-use source_file;
-use {FormatReport, Input, Session};
+use crate::config::{Color, Config, EmitMode, FileName, ReportTactic};
+use crate::formatting::{ModifiedChunk, SourceFile};
+use crate::rustfmt_diff::{make_diff, print_diff, DiffLine, Mismatch, OutputWriter};
+use crate::source_file;
+use crate::{FormatReport, Input, Session};
 
 const DIFF_CONTEXT_SIZE: usize = 3;
 const CONFIGURATIONS_FILE_NAME: &str = "Configurations.md";
@@ -102,7 +101,7 @@ fn verify_config_test_names() {
 // `print_diff` selects the approach not used.
 fn write_message(msg: &str) {
     let mut writer = OutputWriter::new(Color::Auto);
-    writer.writeln(&format!("{}", msg), None);
+    writer.writeln(msg, None);
 }
 
 // Integration tests. The files in the tests/source are formatted and compared
@@ -113,7 +112,7 @@ fn write_message(msg: &str) {
 fn system_tests() {
     // Get all files in the tests/source directory.
     let files = get_test_files(Path::new("tests/source"), true);
-    let (_reports, count, fails) = check_files(files, None);
+    let (_reports, count, fails) = check_files(files, &None);
 
     // Display results.
     println!("Ran {} system tests.", count);
@@ -125,7 +124,7 @@ fn system_tests() {
 #[test]
 fn coverage_tests() {
     let files = get_test_files(Path::new("tests/coverage/source"), true);
-    let (_reports, count, fails) = check_files(files, None);
+    let (_reports, count, fails) = check_files(files, &None);
 
     println!("Ran {} tests in coverage mode.", count);
     assert_eq!(fails, 0, "{} tests failed", fails);
@@ -146,7 +145,9 @@ fn modified_test() {
     let filename = "tests/writemode/source/modified.rs";
     let mut data = Vec::new();
     let mut config = Config::default();
-    config.set().emit_mode(::config::EmitMode::ModifiedLines);
+    config
+        .set()
+        .emit_mode(crate::config::EmitMode::ModifiedLines);
 
     {
         let mut session = Session::new(config, Some(&mut data));
@@ -229,7 +230,7 @@ fn idempotence_tests() {
     }
     // Get all files in the tests/target directory.
     let files = get_test_files(Path::new("tests/target"), true);
-    let (_reports, count, fails) = check_files(files, None);
+    let (_reports, count, fails) = check_files(files, &None);
 
     // Display results.
     println!("Ran {} idempotent tests.", count);
@@ -250,7 +251,7 @@ fn self_tests() {
     }
     files.push(PathBuf::from("src/lib.rs"));
 
-    let (reports, count, fails) = check_files(files, Some(PathBuf::from("rustfmt.toml")));
+    let (reports, count, fails) = check_files(files, &Some(PathBuf::from("rustfmt.toml")));
     let mut warnings = 0;
 
     // Display results.
@@ -280,35 +281,51 @@ fn stdin_formatting_smoke_test() {
         session.format(input).unwrap();
         assert!(session.has_no_errors());
     }
-    //eprintln!("{:?}", );
+
     #[cfg(not(windows))]
     assert_eq!(buf, "fn main() {}\n".as_bytes());
     #[cfg(windows)]
     assert_eq!(buf, "fn main() {}\r\n".as_bytes());
 }
 
-// FIXME(#1990) restore this test
-// #[test]
-// fn stdin_disable_all_formatting_test() {
-//     let input = String::from("fn main() { println!(\"This should not be formatted.\"); }");
-//     let mut child = Command::new("./target/debug/rustfmt")
-//         .stdin(Stdio::piped())
-//         .stdout(Stdio::piped())
-//         .arg("--config-path=./tests/config/disable_all_formatting.toml")
-//         .spawn()
-//         .expect("failed to execute child");
+#[test]
+fn stdin_parser_panic_caught() {
+    // https://github.com/rust-lang/rustfmt/issues/3239
+    for text in ["{", "}"].iter().cloned().map(String::from) {
+        let mut buf = vec![];
+        let mut session = Session::new(Default::default(), Some(&mut buf));
+        let _ = session.format(Input::Text(text));
 
-//     {
-//         let stdin = child.stdin.as_mut().expect("failed to get stdin");
-//         stdin
-//             .write_all(input.as_bytes())
-//             .expect("failed to write stdin");
-//     }
-//     let output = child.wait_with_output().expect("failed to wait on child");
-//     assert!(output.status.success());
-//     assert!(output.stderr.is_empty());
-//     assert_eq!(input, String::from_utf8(output.stdout).unwrap());
-// }
+        assert!(session.has_parsing_errors());
+    }
+}
+
+#[test]
+fn stdin_disable_all_formatting_test() {
+    match option_env!("CFG_RELEASE_CHANNEL") {
+        None | Some("nightly") => {}
+        _ => return, // these tests require nightly
+    }
+    let input = String::from("fn main() { println!(\"This should not be formatted.\"); }");
+    let mut child = Command::new(rustfmt().to_str().unwrap())
+        .stdin(Stdio::piped())
+        .stdout(Stdio::piped())
+        .arg("--config-path=./tests/config/disable_all_formatting.toml")
+        .spawn()
+        .expect("failed to execute child");
+
+    {
+        let stdin = child.stdin.as_mut().expect("failed to get stdin");
+        stdin
+            .write_all(input.as_bytes())
+            .expect("failed to write stdin");
+    }
+
+    let output = child.wait_with_output().expect("failed to wait on child");
+    assert!(output.status.success());
+    assert!(output.stderr.is_empty());
+    assert_eq!(input, String::from_utf8(output.stdout).unwrap());
+}
 
 #[test]
 fn format_lines_errors_are_reported() {
@@ -335,7 +352,7 @@ fn format_lines_errors_are_reported_with_tabs() {
 
 // For each file, run rustfmt and collect the output.
 // Returns the number of files checked and the number of failures.
-fn check_files(files: Vec<PathBuf>, opt_config: Option<PathBuf>) -> (Vec<FormatReport>, u32, u32) {
+fn check_files(files: Vec<PathBuf>, opt_config: &Option<PathBuf>) -> (Vec<FormatReport>, u32, u32) {
     let mut count = 0;
     let mut fails = 0;
     let mut reports = vec![];
@@ -514,7 +531,8 @@ fn read_significant_comments(file_name: &Path) -> HashMap<String, String> {
                         .to_owned(),
                 )
             })
-        }).collect()
+        })
+        .collect()
 }
 
 // Compare output to input.
@@ -743,7 +761,7 @@ impl ConfigCodeBlock {
             .code_block
             .as_ref()
             .unwrap()
-            .split('\n')
+            .lines()
             .nth(0)
             .unwrap_or("")
             == "#![rustfmt::skip]";
@@ -767,7 +785,7 @@ impl ConfigCodeBlock {
         true
     }
 
-    fn has_parsing_errors<T: Write>(&self, session: &Session<T>) -> bool {
+    fn has_parsing_errors<T: Write>(&self, session: &Session<'_, T>) -> bool {
         if session.has_parsing_errors() {
             write_message(&format!(
                 "\u{261d}\u{1f3fd} Cannot format {}:{}",
@@ -882,7 +900,8 @@ fn configuration_snippet_tests() {
         let mut file_iter = BufReader::new(
             fs::File::open(Path::new(CONFIGURATIONS_FILE_NAME))
                 .expect(&format!("Couldn't read file {}", CONFIGURATIONS_FILE_NAME)),
-        ).lines()
+        )
+        .lines()
         .map(|l| l.unwrap())
         .enumerate();
         let mut code_blocks: Vec<ConfigCodeBlock> = Vec::new();
@@ -944,21 +963,32 @@ fn rustfmt() -> PathBuf {
     let mut me = env::current_exe().expect("failed to get current executable");
     me.pop(); // chop of the test name
     me.pop(); // chop off `deps`
+
+    // if we run `cargo test --release` we might only have a release build
+    if cfg!(release) {
+        // ../release/
+        me.pop();
+        me.push("release");
+    }
     me.push("rustfmt");
     assert!(
         me.is_file() || me.with_extension("exe").is_file(),
-        "no rustfmt bin, try running `cargo build` before testing"
+        if cfg!(release) {
+            "no rustfmt bin, try running `cargo build --release` before testing"
+        } else {
+            "no rustfmt bin, try running `cargo build` before testing"
+        }
     );
-    return me;
+    me
 }
 
 #[test]
 fn verify_check_works() {
     let temp_file = make_temp_file("temp_check.rs");
-    assert_cli::Assert::command(&[
-        rustfmt().to_str().unwrap(),
-        "--check",
-        temp_file.path.to_str().unwrap(),
-    ]).succeeds()
-    .unwrap();
+
+    Command::new(rustfmt().to_str().unwrap())
+        .arg("--check")
+        .arg(temp_file.path.to_str().unwrap())
+        .status()
+        .expect("run with check option failed");
 }
