@@ -541,3 +541,24 @@ fn issue_3003() {
     .iter()
     .collect();
 }
+
+fn issue3226() {
+    {
+        {
+            {
+                return Err(
+                    ErrorKind::ManagementInterfaceError("Server exited unexpectedly").into(),
+                );
+            }
+        }
+    }
+    {
+        {
+            {
+                break Err(
+                    ErrorKind::ManagementInterfaceError("Server exited unexpectedlyy").into(),
+                );
+            }
+        }
+    }
+}
